# from IPython.nbformat.v3 import nbpy
from nbformat.v3 import nbpy
from IPython import nbformat as nbf

from io import StringIO


def convert(input_string, output_filename):
    """
    Convert a preprocessed string object into notebook file
    """
    # Read using v3
    with StringIO(input_string) as fin:
        nb = nbpy.read(fin)
    # Write using the most recent version
    with open(output_filename, 'w') as fout:
        nbf.write(nb, fout, version=max(nbf.versions))



import sys
from collections import namedtuple
import tokenize

if sys.version_info[0] == 2:
    TokenInfo = namedtuple('TokenInfo', 'type string start end line')

    def _generate_tokens(readline):
        return map(lambda x: TokenInfo(*x), tokenize.generate_tokens(readline))

else:
    TokenInfo = tokenize.TokenInfo
    _generate_tokens = tokenize.tokenize


def read(filename):
    """
    Read a regular Python file with special formatting and performance
    preprocessing on it.  The result is a string that conforms to the IPython
    notebook version 3 python script format.
    """
    with open(filename, 'rb') as fin:
        token_gen = _generate_tokens(fin.readline)
        cvt_docstr_gen = convert_toplevel_docstring(token_gen)
        nl_gen = fix_newlines(cvt_docstr_gen)
        out = list(nl_gen)

    formatted = tokenize.untokenize(out).decode('utf-8')
    return fix_empty_lines(formatted)


# =============================================================================
#                                   Helpers
# =============================================================================

def convert_toplevel_docstring(tokens):
    for token in tokens:
        # For each string
        
        if 0:
            if token.start[0] > 20: raise
        
        if token.type == tokenize.STRING:
            text = token.string
            # Must be a docstring
            if text.startswith('"""') or text.startswith("'''"):
                startline, startcol = token.start
                # Starting column MUST be 0
                if startcol == 0:
                    endline, endcol = token.end
                    lines = ['# ' + line
                             for line in text.strip('"\' \n').split('\n')]
                    text = '\n'.join(lines)
                    fmt = '# <markdowncell>\n{0}\n# <codecell>'.format(text)
                    res = TokenInfo(type=tokenize.COMMENT,
                                    start=(startline, startcol),
                                    end=(endline, endcol),
                                    string=fmt,
                                    line='#')
                    yield res
                    # To next token
                    continue

        elif token.type == tokenize.COMMENT:
            text = token.string

            # Detect the %% scientific cell
            if text.startswith('#%%'):
                #print("Spyder cell")
                startline, startcol = token.start
                # Starting column MUST be 0
                if startcol == 0:
                    endline, endcol = token.end
                    lines = [line
                             for line in text.strip('"\' \n').split('\n')]
                    assert len(lines) == 1
                    text = '\n'.join(lines)
                    #text = text[3:]
                    text = text.replace("#%%", "") 
                    text = text.lstrip()
                    fmt = '# <markdowncell>\n{0}\n# <codecell>'.format(text)
                    res = TokenInfo(type=tokenize.COMMENT,
                                    start=(startline, startcol),
                                    end=(endline, endcol),
                                    string=fmt,
                                    line='#')
                    yield res
                    # To next token
                    continue  
      
        # Return untouched
        yield token



def fix_newlines(tokens):
    first = True
    curline = 1
    for token in tokens:
        if first:
            first = False
            curline = token.end[0] + 1
        else:
            # Fill NEWLINE token in between
            while curline < token.start[0]:
                yield TokenInfo(type=tokenize.NEWLINE,
                                string='\n',
                                start=(curline, 0),
                                end=(curline, 0),
                                line='\n', )
                curline += 1

            curline = token.end[0] + 1
        yield token


def fix_empty_lines(text):
    def gen():
        for line in text.splitlines():
            if not line.strip():
                # Empty line
                yield ''
            else:
                yield line

    return '\n'.join(gen())



import os
from py2jnb.tools import python_to_notebook

def test_convert():
    print("\nSTART TEST")
    SAMPLE_DIR = os.path.join(os.getcwd(),'samples')

    INPUT_FNAME = 'hello.py'
    BASENAME, _ = os.path.splitext(INPUT_FNAME)
    INPUT_PY = os.path.join(SAMPLE_DIR,INPUT_FNAME)
    print("Input file:",INPUT_PY)
    assert os.path.exists(INPUT_PY)

    OUTPUT_IPYNB = os.path.join(SAMPLE_DIR,BASENAME+".ipynb")
    print("Output file: ", OUTPUT_IPYNB)
    if os.path.exists(OUTPUT_IPYNB):
        os.remove(OUTPUT_IPYNB)

    python_to_notebook(INPUT_PY,OUTPUT_IPYNB)
    assert os.path.exists(OUTPUT_IPYNB)

    # os.remove(OUTPUT_IPYNB)


def test_cells_convert():
    SAMPLE_DIR = os.path.join(os.getcwd(), 'samples')

    INPUT_FNAME = 'hello_cells.py'
    BASENAME, _ = os.path.splitext(INPUT_FNAME)
    INPUT_PY = os.path.join(SAMPLE_DIR, INPUT_FNAME)
    print("Input file:", INPUT_PY)
    assert os.path.exists(INPUT_PY)

    OUTPUT_IPYNB = os.path.join(SAMPLE_DIR, BASENAME + ".ipynb")
    print("Output file: ", OUTPUT_IPYNB)
    if os.path.exists(OUTPUT_IPYNB):
        os.remove(OUTPUT_IPYNB)

    python_to_notebook(INPUT_PY,OUTPUT_IPYNB)
    assert os.path.exists(OUTPUT_IPYNB)

    # os.remove(OUTPUT_IPYNB)


import setuptools

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.md') as changelog_file:
    changelog = changelog_file.read()

requirements = ['nbformat', 'IPython']

# setup_requirements = ['', ]

# test_requirements = ['pytest', ]

setuptools.setup(
    name='py2jnb',
    author="M.Jones, forked from Siu Kwan Lam",
    author_email='jones.0bj3@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
    ],
    description="Utility to turn IPython scripts into Jupyter Notebook format. Forked from py2nb project.",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type='text/markdown',
    # include_package_data=True,
    # keywords='oceandb-elasticsearch-driver',
    packages=setuptools.find_packages(),
    # setup_requires=setup_requirements,
    # test_suite='tests',
    # tests_require=test_requirements,
    url='https://github.com/MarcusJones/py2jnb',
    version='0.0.4',
    # zip_safe=False,
)



import argparse
from py2jnb.tools import python_to_notebook

description = """
Convert literate python script into Jupyter Notebook
"""


def main():
    ap = argparse.ArgumentParser(description=description)
    ap.add_argument("source", help="python filename")
    ap.add_argument("destination", help="output notebook filename")
    args = ap.parse_args()
    # Process
    python_to_notebook(input_filename=args.source,
                       output_filename=args.destination)

if __name__ == '__main__':
    main()



"""
Provide simplified entry points for common task
"""
from .converter import convert
from .reader import read


def python_to_notebook(input_filename, output_filename):
    """
    Convert the given python source file into a properly formatted notebook.
    """
    cvt = read(input_filename)
    convert(cvt, output_filename)
